<?php
// This is called by cron
error_reporting(E_ALL);
ini_set("track_errors", "On");

$now = time();

require_once("/etc/rocketgit/config.php");

$INC = dirname(__FILE__) . "/../inc";
require_once($INC . "/init.inc.php");
require_once($INC . "/log.inc.php");
require_once($INC . "/sql.inc.php");
require_once($INC . "/struct.inc.php");
require_once($INC . "/repo.inc.php");
require_once($INC . "/keys.inc.php");
require_once($INC . "/fixes.inc.php");
require_once($INC . "/ver.php");

rg_log_set_file($rg_log_dir . "/cron.log");
rg_log_set_sid("000000"); // to spread the logs

// locking
rg_lock_or_exit("cron.lock");

rg_log("Start (ver=$rocketgit_version)...");

rg_sql_app("rg-cron");
$db = rg_sql_open($rg_sql);
if ($db === FALSE) {
	rg_log("Cannot connect to database (" . rg_sql_error() . ")!");
	// TODO: inform admin - already by e-mail?
	exit(1);
}

$r = rg_sql_struct_update($db, 0);
if ($r !== TRUE)
	exit(1);

$r = rg_fixes_update($db);
if ($r !== TRUE)
	exit(1);

if ((gmdate("H") == 0) && (gmdate("i") == 3)) {
	while (1) {
		rg_log("Compute repository sizes if dirty...");
		// delete 'dirty' files
		$sql = "SELECT uid, repo_id, master FROM repos";
		$res = rg_sql_query($db, $sql);
		if ($res === FALSE) {
			// TODO: rg_internal_error? it must notify me in case of problems
			rg_log("Cannot run query (" . rg_sql_error() . ")!");
			break;
		}

		while (($row = rg_sql_fetch_array($res))) {
			rg_log("Processing repo " . $row['repo_id'] . "...");
			$repo_path = rg_repo_path_by_id($row['uid'], $row['repo_id']);

			$all_files = $row['master'] == 0 ? TRUE : FALSE;
			$disk_used = rg_repo_size($repo_path, $all_files);
			if ($disk_used === FALSE) {
				rg_log("Could not open dir!");
			} else {
				$disk_used_mb = intval($disk_used / 1024 / 1024);
				$sql = "UPDATE repos SET disk_used_mb = $disk_used_mb"
					. " WHERE repo_id = " . $row['repo_id'];
				$res2 = rg_sql_query($db, $sql);
				if ($res2 === FALSE) {
					rg_log("Cannot run query!");
					break;
				}

				rg_sql_free_result($res2);
			}
		}
		rg_sql_free_result($res);
		break;
	}

	while (1) {
		rg_log("Compute repository sizes per user...");
		$sql = "SELECT SUM(disk_used_mb) AS disk_used_mb, uid"
			. " FROM repos"
			. " GROUP BY uid";
		$res = rg_sql_query($db, $sql);
		if ($res === FALSE) {
			rg_log("Cannot run query (" . rg_sql_error() . ")!");
			break;
		}

		while (($row = rg_sql_fetch_array($res))) {
			$sql = "UPDATE users"
				. " SET disk_used_mb = " . $row['disk_used_mb']
				. " WHERE uid = " . $row['uid'];
			$res2 = rg_sql_query($db, $sql);
			rg_sql_free_result($res2);
		}
		rg_sql_free_result($res);
		break;
	}
}

if (gmdate("H") == 3) {
	rg_log("Clean old forget_pass entries...");
	$sql = "DELETE FROM forgot_pass WHERE expire < $now";
	$res = rg_sql_query($db, $sql);
	rg_sql_free_result($res);
}

if (gmdate("H") == 1) {
	rg_log("Clean old tokens...");
	$sql = "DELETE FROM tokens WHERE expire < $now";
	$res = rg_sql_query($db, $sql);
	rg_sql_free_result($res);
}

if (gmdate("H") == 1) {
	rg_log("Clean old sess entries...");
	$sql = "DELETE FROM sess WHERE expire < $now";
	$res = rg_sql_query($db, $sql);
	rg_sql_free_result($res);
}

rg_log_cron();

if (gmdate("H") == 4)
	rg_sql_struct_slaves_update($db);


rg_log("Done!");
?>
