<?php
error_reporting(E_ALL | E_STRICT);
ini_set("track_errors", "On");

$INC = dirname(__FILE__) . "/../inc";
require_once(dirname(__FILE__) . "/config.php");
require_once($INC . "/init.inc.php");
require_once($INC . "/log.inc.php");
require_once($INC . "/sql.inc.php");

rg_log_set_file("sql.log");

require_once("common.php");

echo "db: drop 'test' table...\n";
$sql = "DROP TABLE IF EXISTS test";
$res = rg_sql_query($db, $sql);
if ($res === FALSE) {
	echo "Cannot create table 'test' (" . rg_sql_error() . ")!\n";
	exit(1);
}

echo "db: test creation of a table...\n";
$sql = "CREATE TABLE test (id TEXT PRIMARY KEY"
	. ", f1 TEXT DEFAULT '', f2 TEXT DEFAULT '')";
$res = rg_sql_query($db, $sql);
if ($res === FALSE) {
	echo "Cannot create table 'test' (" . rg_sql_error() . ")!\n";
	exit(1);
}

echo "db: test insert...\n";
$sql = "INSERT INTO test (id) VALUES ('aaa')";
$res = rg_sql_query($db, $sql);
if ($res === FALSE) {
	echo "Cannot insert!\n";
	exit(1);
}

echo "db: test insert with the same key...\n";
$sql = "INSERT INTO test (id) VALUES ('aaa')";
$res = @rg_sql_query($db, $sql);
if ($res !== FALSE) {
	echo "I can do double insert, not good!\n";
	exit(1);
}

echo "db: test delete...\n";
$sql = "DELETE FROM test WHERE id = 'aaa'";
$res = rg_sql_query($db, $sql);
if ($res === FALSE) {
	echo "Cannot delete!\n";
	exit(1);
}

echo "db: test prepare with named values...\n";
$sql = "INSERT INTO test(id, f1, f2) VALUES (@@id@@, @@f@@, @@f@@)";
$params = array("id" => "myid", "f" => "value", "junk" => "aaa");
$res = rg_sql_query_params($db, $sql, $params);
if ($res === FALSE) {
	echo "Cannot insert into test using @@x@@!\n";
	exit(1);
}
$sql = "SELECT * FROM test WHERE id = 'myid'";
$res = rg_sql_query($db, $sql);
$row = rg_sql_fetch_array($res);
if (strcmp($row['f1'], "value") != 0) {
	echo "Seems that insert with @@x@@ are not working!\n";
	exit(1);
}

// TODO: test rg_sql_last_id

rg_sql_close($db);

echo "db: OK!\n";
?>
